
source("modules/data_load/dates2load.R")
source("modules/data_load/regrid.R")
# source("modules/data_load/s2dv_cube.R")
# source("recipe.R")

# Load required libraries
source("tools/libs.R")

# RECIPE FOR TESTING
# -------------------------------------------------------------------------------------------
## TODO: Get only the last part of the path as the recipe$filename? 
recipe <- read_yaml("/esarchive/scratch/vagudets/repos/auto-s2s/modules/data_load/recipe_2.yml")
args <- NULL; args[1] <- "/esarchive/scratch/vagudets/repos/auto-s2s/modules/data_load/recipe_2.yml"
recipe$filename <- args[1]
# Create output folder and log:
logger <- prepare_outputs(recipe = recipe) 
folder <- logger$foldername
log_file <- logger$logname
logger <- logger$logger
# -------------------------------------------------------------------------------------------

# Set params -----------------------------------------
hcst.inityear <- recipe$Analysis$Time$hcst_start
hcst.endyear <- recipe$Analysis$Time$hcst_end
ltmin <- recipe$Analysis$Time$leadtimemin
ltmax <- recipe$Analysis$Time$leadtimemax
lats.min <- recipe$Analysis$Region$latmin
lats.max <- recipe$Analysis$Region$latmax
lons.min <- recipe$Analysis$Region$lonmin
lons.max <- recipe$Analysis$Region$lonmax
ref.name <- recipe$Analysis$Datasets$Reference$name
exp.name <- recipe$Analysis$Datasets$System$name

variable <- recipe$Analysis$Variables$name
store.freq <- recipe$Analysis$Variables$freq

## TODO: Examine this verifications part, verify if it's necessary
# stream <- verifications$stream
# sdates <- verifications$fcst.sdate

## TODO: define fcst.name
##fcst.name <- recipe$Analysis$Datasets$System[[sys]]$name

# get sdates array
sdates <- dates2load(recipe, logger)

# get esarchive datasets dict:
archive <- read_yaml(paste0(recipe$Run$code_dir, "conf/archive.yml"))$archive
exp_descrip <- archive[[exp.name]]

freq.hcst <- unlist(exp_descrip[[store.freq]][variable])
reference_descrip <- archive[[ref.name]]
freq.obs <- unlist(reference_descrip[[store.freq]][variable])
obs.dir <- reference_descrip$src
fcst.dir <- exp_descrip$src 
hcst.dir <- exp_descrip$src
fcst.nmember <- exp_descrip$nmember$fcst
hcst.nmember <- exp_descrip$nmember$hcst

## TODO: it is necessary?
##if ("accum" %in% names(reference_descrip)) {
##  accum <- unlist(reference_descrip$accum[store.freq][[1]])
##} else {
##  accum <- FALSE
##}

# -----------
obs.path <- paste0(archive$src,
            obs.dir, store.freq, "/$var$",
            reference_descrip[[store.freq]][[variable]], "$var$_$file_date$.nc")

hcst.path <- paste0(archive$src,
             hcst.dir, store.freq, "/$var$",
             exp_descrip[[store.freq]][[variable]], "$var$_$file_date$.nc")

# Regrid:
#-------------------------------------------------------------------
regrid_params <- get_regrid_params(recipe, archive)

# Longitude sort
#-------------------------------------------------------------------
if (lons.min >= 0) {
  circularsort <- CircularSort(0, 360)
} else {
  circularsort <- CircularSort(-180, 180)
}

# Hindcast timeseries load 
#-------------------------------------------------------------------
## TODO: Adapt to daily data 
hcst <- Start(dat = hcst.path,
              var = variable,
              file_date = sdates$hcst,
              time = indices(ltmin:ltmax),
              latitude = values(list(lats.min, lats.max)),
              latitude_reorder = Sort(),
              longitude = values(list(lons.min, lons.max)),
              longitude_reorder = circularsort,
              transform = regrid_params$fcst.transform,
              transform_params = list(grid = regrid_params$fcst.gridtype,
                                      method = regrid_params$fcst.gridmethod),
              transform_vars = c('latitude', 'longitude'),
              synonims = list(latitude = c('lat', 'latitude'),
                              longitude = c('lon', 'longitude'),
                              member = c('ensemble')), 
              member = indices(1:hcst.nmember),
              return_vars = list(latitude = 'dat',
                                 longitude = 'dat',
                                 time = 'file_date'),
              split_multiselected_dims = TRUE,
              retrieve = TRUE)

# Get forecast dates 
dates <- attr(hcst, 'Variables')$common$time
dates_file <- sapply(dates, format, '%Y%m%d')
dates_file <- format(as.Date(dates_file, '%Y%m%d'), "%Y%m")
dim(dates_file) <- dim(Subset(hcst, 
                              along=c('dat','var',
                              'latitude', 'longitude', 'member'), 
                              list(1,1,1,1,1), drop="selected"))

# Reference timeseries load 
#-------------------------------------------------------------------
## TODO: Not working for daily obs

obs <- Start(dat = obs.path,		  
             var = variable,
             file_date = dates_file,
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),
             longitude = values(list(lons.min, lons.max)),
             longitude_reorder = circularsort,
             synonims = list(latitude = c('lat', 'latitude'),
                             longitude = c('lon', 'longitude')),
             transform = regrid_params$obs.transform,
             transform_params = list(grid = regrid_params$obs.gridtype,
                                     method = regrid_params$obs.gridmethod),
             transform_vars = c('latitude', 'longitude'),
	     return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = 'file_date'),
	     split_multiselected_dims = TRUE,
	     retrieve = TRUE)

# Conversion from startR_array to s2dv_array
#-------------------------------------------------------------------

attr2array <- function(attr){
  return(array(as.vector(attr), dim(attr)))
}


startR_to_s2dv <- function(startR_array){

  dates_dims <- c("syear","time")
  dates_start <- attr(startR_array, 'Variables')$common$time
  dates_end <- attr(startR_array, 'Variables')$common$time
  names(dim(dates_start)) <- dates_dims
  names(dim(dates_end)) <- dates_dims

  s2dv_object <- s2dv_cube(data = attr2array(startR_array), 
                           lon = attr2array(attr(startR_array, 'Variables')$dat1$longitude),
                           lat = attr2array(attr(startR_array, 'Variables')$dat1$latitude),
                           Variable = list(varName = names(attr(startR_array,
							        'Variables')$common)[2],
                                           level = NULL),
                           Dates = list(start = dates_start,
                                        end = dates_end),
                           when = Sys.time(), 
                           source_files = attr2array(attr(startR_array, "Files"))
                           #Datasets = list(exp1 = list(InitializationsDates = list(Member_1 = "01011990", 
                           #                                                        Members = "Member_1")))
                          ) 
  
  return(s2dv_object)

}

hcst <- startR_to_s2dv(hcst)
obs <- startR_to_s2dv(obs)

## TODO: new files checker?
#hcst.NA_files <- c(attributes(hcst)$NotFoundFiles)
#hcst.NA_files <- hcst.NA_files[!is.na(hcst.NA_files)]
#try(hcst.NA_files <- hcst.NA_files[order(hcst.NA_files)])

dates_file <- paste0(dates_file, '01')
dim(dates_file) <- dim(Subset(hcst,
                              along=c('dat','var',
                              'latitude', 'longitude', 'member'),
                              list(1,1,1,1,1), drop="selected"))

file_dates <- paste0(file_dates, '01')
dim(file_dates) <- dim(Subset(hcst,
                              along=c('dat','var',
                              'latitude', 'longitude', 'member', 'time'),
                              list(1,1,1,1,1,1), drop="selected"))



obs.NA_dates.ind <- Apply(obs, 
                          fun=(function(x){ all(is.na(x))}), 
                          target_dims=c('time', 'latitude', 'longitude'))[[1]]
obs.NA_dates <- dates_file[obs.NA_dates.ind]
obs.NA_dates <- obs.NA_dates[order(obs.NA_dates)]
obs.NA_files <- paste0(obs.dir, store.freq,"/",variable,"_",
                       freq.obs,"obs.grid","/",variable,"_",obs.NA_dates,".nc")

if (any(is.na(hcst))){
  fatal(logger,
    paste(" ERROR: MISSING HCST VALUES FOUND DURING LOADING # ",
          " ################################################# ",
          " ######           MISSING FILES               #### ",
          " ################################################# ",
          "hcst files:",
          hcst.NA_files,
          " ################################################# ",
          " ################################################# ",
          sep="\n"))
  quit(status = 1)
}

if (any(is.na(obs)) && !identical(obs.NA_dates,character(0))){
  fatal(logger,
    paste(" ERROR: MISSING OBS VALUES FOUND DURING  LOADING # ",
          " ################################################# ",
          " ######           MISSING FILES               #### ",
          " ################################################# ",
          "obs files:",
          obs.NA_files,
          " ################################################# ",
          " ################################################# ",
          sep="\n"))
  quit(status=1)
}

info(logger,
     "######### DATA LOADING COMPLETED SUCCESFULLY ##############")

default_dims <- c(dat = 1, var = 1, sweek = 1, 
                  sday = 1, syear = 1, time = 1, 
                  latitude = 1, longitude = 1, member = 1)

default_dims[names(dim(obs))] <- dim(obs)
dim(obs) <- default_dims

lon <- attr(obs, 'Variables')$dat1$longitude
lat <- attr(obs, 'Variables')$dat1$latitude
hcst.times <- attr(hcst, 'Variables')$common$time
hcst.times <- sort(unique(sapply(as.character(hcst.times),
                   substr, 1, 10)))

#obs<-Subset(obs,c('dat','var'),list(1,1),drop='selected')
#hcst<-Subset(hcst,c('dat','var'),list(1,1),drop='selected')
#if (stream == "fcst"){ 
#  fcst<-Subset(fcst,c('dat','var'),list(1,1),drop='selected')
#}

#filters negative values in accum vars
if (accum){
   obs[obs < 0 ] <- 0
   hcst[hcst < 0 ] <- 0
}

sdates.hcst <- file_dates
leadtimes.hcst <- dates_file

